#!/usr/bin/perl -T
# juniper.pl  -  Juniper Router Script  - (c) 2012, Cyber Operations, Inc.

use strict;
my($cfgfile) = shift(@ARGV);
my(%Config);


sub ParseConfigFile 
{
    my($config_line, $Name, $Value);

    my($File) = @_;

    if (!open (CONFIG, "$File")) 
	{
		return; # should always have a config file
	}

    while (<CONFIG>) 
	{
        $config_line=$_;
        chop ($config_line);          # Get rid of the trailling \n
        $config_line =~ s/^\s*//;     # Remove spaces at the start of the line
        $config_line =~ s/\s*$//;     # Remove spaces at the end of the line
        if ( ($config_line !~ /^#/) && ($config_line ne "") )
		{														  # Ignore lines starting with # and blank lines
            ($Name, $Value) = split (/=/, $config_line);          # Split each line into name value pairs
        	$Name =~ s/^\s*//;     # Remove spaces at the start of the line
        	$Name =~ s/\s*$//;     # Remove spaces at the end of the line
        	$Value =~ s/^\s*//;     # Remove spaces at the start of the line
        	$Value =~ s/\s*$//;     # Remove spaces at the end of the line
            $Config{lc($Name)} = $Value;                             # Create a hash of the name value pairs
        }
    }

    close(CONFIG);
}

sub Get()
{
	my($key) = @_;
	my($value) = $Config{lc($key)};

	# handle quoting and backslash escapes

	# strip out of quotations within leading/trailing whitespace
	$value =~ s/^\s*["'](.*)["']\s*$/$1/;
	
	# translate tab and newline escapes
	$value =~ s/\\t/\t/g;
	$value =~ s/\\n/\n/g;
	# translate escaped quotes
	$value =~ s/\\\'/\'/g;
	$value =~ s/\\\"/\"/g;

	return $value;
}

sub BuildNatDNS
{
	my($term, $prefixlist, $ret);
	if($Config{"ingress"} eq '1')
	{
		$term = "DNS-TO-PROXY";
	}
	else
	{
		$term = "PROXY-TO-DNS";
	}

	$prefixlist  = $Config{"nat_dns" };


	if( $prefixlist ne "" )
	{
		$ret = <<END;
		term $term {
			from {
				destination-prefix-list {
					$prefixlist;
				}
			}
			then accept;
		}
END
	}

	return ($ret);
}


# build the juniper config entries for the filter to send TCP RST for the specified mail server addresses
sub BuildMailRedirect
{
	my($emailips);

	$emailips = $Config{"emailips"};

	my(@maddr) = split(/,/, $emailips);
	my $numaddr = @maddr;
	my $result= "";
	if( $numaddr > 0 )
	{
		foreach( @maddr )
		{
			$_ =~ s/^\s+|\s+$//g ;
		}

		$result .= "\t\tterm MAILRST {\n";
		$result .= "\t\t\tfrom {\n";
		$result .= "\t\t\t\tdestination-port [25];\n";
		$result .= "\t\t\t\tdestination-address {\n";
		foreach( @maddr )
		{
			$result .= "\t\t\t\t\t" . $_ . ";\n";
		}
		$result .= "\t\t\t\t}\n";
		$result .= "\t\t\t}\n";
		$result .= "\t\t\tthen reject tcp-reset;\n";
		$result .= "\t\t}\n";
	}

	return $result;
}


########## main ###############
	
	#parse the config file first
	&ParseConfigFile ($cfgfile);

	my($dnsudp) = 0;
	# load the mail server addresses from the config file
	my($emailfix) = BuildMailRedirect();
	my($natdns) = BuildNatDNS();
	my($dnsfrom) = &Get("dnsfrom");
	my($dnsthenA) = &Get("dnsthenA");
	my($dnsthenD) = &Get("dnsthenD");
	my($accept) = &Get("accept");
	my($icmpfrom) = &Get("icmpfrom");
	my($icmpthenA) = &Get("icmpthenA");
	my($icmpthenD) = &Get("icmpthenD");
	my($discard) = &Get("discard");
	my($ingress) = ($Config{"ingress"} eq '1');

	my($output);

	foreach (<STDIN>)
	{
		# save if udp or not for DNS purposes
		if( $_ =~ /protocol/ )
		{
			if( $_ =~ /udp/ )
			{
				$dnsudp = 1;
			}
			else
			{
				$dnsudp = 0;
			}
		}

		#### for email stuff
		# match the filter section
		# (whitespace-filter-whitespace-filtername-whitespace-brace
		if($_ =~ /^*\sfilter*\s.*\s\{/ )
		{
			$output .= $_; # actual values
			# email statement only goes in on ingress filters
			if($ingress)
			{
				$output .= $emailfix;
			}
			$output .= $natdns;
		}
		elsif($_ =~ /term\s.*ACL1\s*{/ )
		{	
			$output .= "interface-specific;\n";
			$output .= $_;
		}
		elsif($_ =~ /\s.*\/\*DNS-FROM\*\//)
		{
			#always add on egress but only on udp dns on ingress
			if( $dnsudp || !$ingress)
			{
				$output .= $dnsfrom;
			}
		}
		elsif($_ =~ /\s.*accept;\/\*DNS-THEN\*\//)
		{
			# this is part of our fix for DNS policing
			# so it will only apply to requests and not replies
			#always add on egress but only on udp dns on ingress
			if( $dnsudp || !$ingress)
			{
				$output .= $dnsthenA;
			}
			else
			{
				$output .= $accept;
			}
		}
		elsif($_ =~ /\s.*discard;\/\*DNS-THEN\*\//)
		{
			$output .= $dnsthenD;
		}
		elsif($_ =~ /\s.*\/\*ICMP-FROM\*\//)
		{
			$output .= $icmpfrom;
		}		
		elsif($_ =~ /\s.*accept;\/\*ICMP-THEN\*\//)
		{
			$output .= $icmpthenA;
		}
		elsif($_ =~ /\s.*discard;\/\*ICMP-THEN\*\//)
		{
			$output .= $icmpthenD;
		}
		elsif($_ =~ /\s.*accept;/)
		{
			$output .= $accept;
		}
		elsif($_ =~ /\s.*discard;/)
		{
			$output .= $discard;
		}
		else
		{
			$output .= $_;
		}	
	}

	print $output;

exit(0);


